import unittest
from typing import Tuple, Dict
from uuid import uuid4

from shinobi_client.shinobi_controller import ShinobiController, Shinobi
from shinobi_client.user import ShinobiUserOrm


def _create_email_and_password() -> Tuple[str, str]:
    """
    Create email and password.
    :return: email and password tuple
    """
    random_string = str(uuid4())
    return f"{random_string}@example.com", random_string


class TestShinobiUserOrm(unittest.TestCase):
    """
    Tests for `ShinobiUserOrm`.

    Shinobi takes a reasonable amount of time to setup so sharing between all tests in class (at the cost of reduced
    test isolation).
    """
    _shinobi_controller: ShinobiController = None

    @classmethod
    def setUpClass(cls):
        cls._shinobi_controller = ShinobiController()
        cls._shinobi = cls._shinobi_controller.start()

    @classmethod
    def tearDownClass(cls):
        cls._shinobi_controller.stop()

    def setUp(self):
        self._shinobi: Shinobi = TestShinobiUserOrm._shinobi
        self.user_orm = ShinobiUserOrm(
            self._shinobi.host,
            int(self._shinobi.port),
            self._shinobi.super_user_token,
        )

    def test_create(self):
        email, password = _create_email_and_password()
        self.user_orm.create(email, password)
        user = self.user_orm.get(email)
        self.assertEqual(email, user["mail"])

    def test_create_and_verify(self):
        email, password = _create_email_and_password()
        self.user_orm.create(email, password, verify_create=True)
        user = self.user_orm.get(email)
        self.assertEqual(email, user["mail"])

    def test_get(self):
        user = self._create_user()
        retrieved_user = self.user_orm.get(user["email"])
        self.assertEqual(user["mail"], retrieved_user["mail"])

    def test_get_when_does_not_exist(self):
        user = self.user_orm.get("example@doesnotexist.com")
        self.assertIsNone(user)

    def test_get_all(self):
        emails = []
        for i in range(3):
            emails.append(self._create_user()["mail"])

        matched_users = tuple(filter(lambda user: user["mail"] in emails, self.user_orm.get_all()))
        self.assertEqual(len(emails), len(matched_users))

    def test_modify(self):
        user = self._create_user()
        _, password = _create_email_and_password()
        assert user["pass"] != password
        changed = self.user_orm.modify(user["email"], password=password)
        self.assertTrue(changed is None or changed is True)

    def test_modify_when_user_not_exist(self):
        email, password = _create_email_and_password()
        self.assertRaises(ValueError, self.user_orm.modify, email, password=password)

    def test_delete(self):
        user = self._create_user()
        self.user_orm.delete(user["email"])
        self.assertIsNone(self.user_orm.get(user["email"]))

    def test_delete_and_verify(self):
        user = self._create_user()
        self.user_orm.delete(user["email"], verify_delete=True)
        self.assertIsNone(self.user_orm.get(user["email"]))

    def test_delete_when_user_not_exist(self):
        email, _ = _create_email_and_password()
        self.assertFalse(self.user_orm.delete(email))

    def _create_user(self) -> Dict:
        """
        Creates a user.
        :return: model of the created user
        """
        email, password = _create_email_and_password()
        return self.user_orm.create(email, password, verify_create=False)


if __name__ == "__main__":
    unittest.main()



import unittest

import requests

from shinobi_client.shinobi_controller import ShinobiController, start_shinobi


class TestShinobiController(unittest.TestCase):
    """
    Tests for `ShinobiController`.
    """
    def setUp(self):
        self._shinobi_controller = ShinobiController()

    def tearDown(self):
        self._shinobi_controller.stop()

    def test_start(self):
        shinobi = self._shinobi_controller.start()
        home_page_request = requests.get(shinobi.url)
        self.assertTrue(home_page_request.ok)

    def test_stop(self):
        shinobi = self._shinobi_controller.start()
        self._shinobi_controller.stop()
        self.assertRaises(requests.exceptions.ConnectionError, requests.get, shinobi.url)

    def test_stop_when_not_started(self):
        self._shinobi_controller.stop()


class TestStartShinobi(unittest.TestCase):
    """
    Tests for `start_shinobi`.
    """
    def test_use_in_context(self):
        with start_shinobi() as shinobi:
            home_page_request = requests.get(shinobi.url)
            self.assertTrue(home_page_request.ok)


if __name__ == "__main__":
    unittest.main()



import json
from copy import deepcopy
from typing import Optional, Dict, Tuple
import requests
from requests import Response
from logzero import logger


class ShinobiUserOrm:
    """
    Shinobi user ORM.
    """
    SUPPORTED_MODIFIABLE_PROPERTIES = {"password"}

    @staticmethod
    def _create_improved_user_entry(user: Dict) -> Dict:
        """
        Creates a copy of the given user details with extra information.
        :param user: user details to improve
        :return: improved user details
        """
        user = deepcopy(user)
        user["email"] = user["mail"]
        return user

    @staticmethod
    def _raise_if_errors(shinobi_response: Response):
        """
        Raises an exception if the response from Shinobi indicated there were errors.
        :param shinobi_response: the response from Shinobi
        """
        shinobi_response.raise_for_status()
        json_response = shinobi_response.json()
        if not json_response["ok"]:
            # Yes, the API returns a non-400 when everything is not ok...
            raise RuntimeError(json_response["msg"])

    def __init__(self, host: str, port: int, super_user_token: str):
        """
        Constructor.
        :param host: location of the Shinboi host
        :param port: port Shinobi is running on
        :param super_user_token: a Shinobi super-user token
        """
        self.host = host
        self.port = port
        self.super_user_token = super_user_token

    def get(self, email: str) -> Optional[Dict]:
        """
        Gets details about the user with the given email address.
        :param email: user's email address
        :return: details about user else `None` if the user does not exist
        """
        # XXX: For some reason, Shinobi doesn't have an endpoint to query an individual user
        users = self.get_all()
        matched_users = tuple(filter(lambda user: user["mail"] == email, users))
        assert len(matched_users) <= 1, f"More than one user found with the email address: {email}"
        if len(matched_users) == 1:
            return ShinobiUserOrm._create_improved_user_entry(matched_users[0])
        else:
            return None

    def get_all(self) -> Tuple:
        """
        Gets details about all users.
        :return: tuple where each element contains details about a specific user
        """
        response = requests.get(
            f"http://{self.host}:{self.port}/super/{self.super_user_token}/accounts/list")
        ShinobiUserOrm._raise_if_errors(response)
        return tuple(ShinobiUserOrm._create_improved_user_entry(user) for user in response.json()["users"])

    def create(self, email: str, password: str, verify_create: bool = False) -> Dict:
        """
        Creates a user with the given details.
        :param email: email address of the user
        :param password: password for the user
        :param verify_create: whether to verify that the user has been successfully created
        :return: details about created user
        """
        # Not trusting Shinobi's API to give back anything useful if the user already exists
        if self.get(email):
            raise ValueError(f"User with email \"{email}\" already exists")

        # The required post does not align with the API documentation (https://shinobi.video/docs/api)
        # Exploiting the undocumented API successfully used by UI.
        # See source: https://gitlab.com/Shinobi-Systems/Shinobi/-/blob/dev/libs/webServerSuperPaths.js
        data = {
            "mail": email,
            "pass": password,
            "password_again": password,
            "details": json.dumps({
                "factorAuth": "0", "size": "", "days": "", "event_days": "", "log_days": "", "max_camera": "",
                "permissions": "all", "edit_size": "1", "edit_days": "1", "edit_event_days": "1",
                "edit_log_days": "1", "use_admin": "1", "use_aws_s3": "1", "use_whcs": "1", "use_sftp": "1",
                "use_webdav": "1", "use_discordbot": "1", "use_ldap": "1", "aws_use_global": "0",
                "b2_use_global": "0", "webdav_use_global": "0"})
        }
        response = requests.post(
            f"http://{self.host}:{self.port}/super/{self.super_user_token}/accounts/registerAdmin",
            json=dict(data=data))
        ShinobiUserOrm._raise_if_errors(response)
        create_user = response.json()

        # This is worth doing as Shinobi's API is all over the place - it happily returns OK for invalid requests
        if verify_create:
            assert self.get(email), "Unable to verify created user"

        return ShinobiUserOrm._create_improved_user_entry(create_user["user"])

    def modify(self, email: str, **kwargs) -> Optional[bool]:
        """
        Modify a user.
        :param email: the email address of the user to modify.
        :param kwargs: new property values (currently supported: `password`).
        :return: whether the user was modified or `None` if not able to confirm
        """
        unsupported_properties = set(kwargs.keys()) - {"mail"} - self.__class__.SUPPORTED_MODIFIABLE_PROPERTIES
        if len(unsupported_properties) > 0:
            raise NotImplementedError(f"Cannot modify user properties: {unsupported_properties}")

        existing_user = self.get(email)
        if existing_user is None:
            raise ValueError(f"Cannot modify user as they do not exist: {email}")

        data = {
            "mail": email,
            "pass": kwargs["password"],
            "password_again": kwargs["password"],
        }
        account = {
            "mail": email,
            "uid": existing_user["uid"],
            "ke": existing_user["ke"]
        }
        response = requests.post(
            f"http://{self.host}:{self.port}/super/{self.super_user_token}/accounts/editAdmin",
            json=dict(data=data, account=account))
        ShinobiUserOrm._raise_if_errors(response)

        rows_changed = response.json().get("rowsChanged")
        if rows_changed is None:
            logger.info("Shinobi did not return information on whether the user has been changed")
            return None
        return rows_changed == 1

    def delete(self, email: str, verify_delete: bool = False) -> bool:
        """
        Deletes the user with the given email address.
        :param email: email address of user
        :param verify_delete: whether to confirm that the user has been deleted
        :return: `True` if the user has been deleted, else `False` if they haven't because they didn't exist
        """
        user = self.get(email)
        if user is None:
            return False

        account = {
            "uid": user["uid"],
            "ke": user["ke"],
            "mail": email
        }

        # Odd interface, defined here:
        # https://gitlab.com/Shinobi-Systems/Shinobi/-/blob/dev/libs/webServerSuperPaths.js#L385
        response = requests.post(
            f"http://{self.host}:{self.port}/super/{self.super_user_token}/accounts/deleteAdmin",
            json=dict(account=account))
        ShinobiUserOrm._raise_if_errors(response)

        if verify_delete:
            assert self.get(email) is None, f"User with email \"{email}\" was not deleted"

        return True


if __name__ == "__main__":
    import fire
    fire.Fire(ShinobiUserOrm)



import os
import subprocess
from dataclasses import dataclass
from pathlib import Path
from tempfile import mkdtemp
from time import sleep
from typing import Callable, Optional, Dict

import git
import requests
from get_port import find_free_port
from uuid import uuid4

DEFAULT_DOCKER_SHINOBI_GIT_REPO_URL = "https://github.com/colin-nolan/docker-shinobi"
DEFAULT_DOCKER_SHINOBI_GIT_REPO_BRANCH = "master"


def _requires_docker_shinobi(caller: Callable):
    def wrapper(self: "ShinobiController", *args, **kwargs):
        self._clone_docker_shinobi()
        return caller(self, *args, **kwargs)

    return wrapper


class ShinobiAlreadyRunningError(RuntimeError):
    """
    Error raised if it is a problem that Shinobi is already running.
    """


@dataclass
class Shinobi:
    """
    Model of Shinobi installation.
    """
    super_user_email: str
    super_user_password: str
    super_user_token: str
    port: str
    host: str

    @property
    def url(self) -> str:
        return f"http://{self.host}:{self.port}"


class ShinobiController:
    """
    Controls the running of a Shinobi installation from Python.

    Designed for temporarily running Shinobi, e.g. for testing.
    """
    @staticmethod
    def _wait_for_start(shinobi: Shinobi):
        """
        Blocks until service is ready.
        :param shinobi: started shinobi
        """
        interval = 0.05
        while True:
            try:
                requests.get(shinobi.url)
                return
            except requests.exceptions.ConnectionError:
                sleep(interval)
                interval += 0.01

    @property
    def _temp_directory(self) -> str:
        if self.__temp_directory is None:
            self.__temp_directory = mkdtemp(prefix=f"{self.__class__.__name__}-", dir=self.temp_root_location)
        return self.__temp_directory

    @property
    def _shinobi_directory(self) -> str:
        return os.path.join(self._temp_directory, os.path.basename(self.docker_shinobi_git_repo_url))

    def __init__(self, docker_shinobi_git_repo_url: str = DEFAULT_DOCKER_SHINOBI_GIT_REPO_URL,
                 docker_shinobi_git_repo_branch: str = DEFAULT_DOCKER_SHINOBI_GIT_REPO_BRANCH,
                 temp_root_location: str = "/tmp"):
        """
        Constructor.
        :param docker_shinobi_git_repo_url: URL of the Git repository that
        :param temp_root_location: location in which temp files are created (must be mountable using Docker)
        """
        self.temp_root_location = temp_root_location
        self.docker_shinobi_git_repo_url = docker_shinobi_git_repo_url
        self.docker_shinobi_git_repo_branch = docker_shinobi_git_repo_branch
        self.__temp_directory: Optional[str] = None
        self._stop = None

    def __enter__(self) -> Shinobi:
        return self.start()

    def __exit__(self, *args):
        self.stop()

    @_requires_docker_shinobi
    def start(self) -> Shinobi:
        """
        Starts an installation of Shinobi.
        :return: details about the started installation
        :raises ShinobiAlreadyRunningError: if Shinobi is already running
        """
        if self._stop:
            raise ShinobiAlreadyRunningError()

        environment = self._create_run_environment()

        self._stop = lambda: subprocess.check_output(
            ["docker-compose", "down"],
            cwd=self._shinobi_directory, env=environment)

        subprocess.check_output(
            ["docker-compose", "up", "--build", "--detach", "--renew-anon-volumes"],
            cwd=self._shinobi_directory, env=environment)
        shinobi = Shinobi(
            super_user_email=environment["SHINOBI_SUPER_USER_EMAIL"],
            super_user_password=environment["SHINOBI_SUPER_USER_PASSWORD"],
            super_user_token=environment["SHINOBI_SUPER_USER_TOKEN"],
            port=environment["SHINOBI_HOST_PORT"],
            # XXX: hardcoding assumption that the Docker daemon is on the host
            host="0.0.0.0"
        )
        ShinobiController._wait_for_start(shinobi)

        return shinobi

    def stop(self):
        """
        Stops any running instance of a Shinobi installation.
        :return: whether an installation was stopped
        """
        if not self._stop:
            return False
        self._stop()
        self._stop = None
        return True

    def _create_run_environment(self) -> Dict[str, str]:
        """
        Creates the environment in which the containerised Shinobi can run.
        :return: environment
        """
        # Mac does have/allow mounting of /etc/timezone and /etc/localtime so faking them
        timezone_file_location = os.path.join(self._temp_directory, "timezone")
        Path(timezone_file_location).touch()
        localtime_file_location = os.path.join(self._temp_directory, "localtime")
        Path(localtime_file_location).touch()

        port, port_find_error = find_free_port()
        if port_find_error:
            raise RuntimeError(f"Error finding free port: {port_find_error}")

        return dict(
            **os.environ,
            SHINOBI_VIDEO_LOCATION=os.path.join(self._temp_directory, "videos"),
            MYSQL_USER_PASSWORD=str(uuid4()),
            MYSQL_ROOT_PASSWORD=str(uuid4()),
            SHINOBI_SUPER_USER_EMAIL=str(uuid4()),
            SHINOBI_SUPER_USER_PASSWORD=str(uuid4()),
            SHINOBI_SUPER_USER_TOKEN=str(uuid4()),
            SHINOBI_DATA_LOCATION=os.path.join(self._temp_directory, "data"),
            SHINOBI_TIMEZONE=timezone_file_location,
            SHINOBI_LOCALTIME=localtime_file_location,
            SHINOBI_HOST_PORT=str(port)
        )

    def _clone_docker_shinobi(self):
        """
        Clones repository for running containerised Shinobi.
        """
        if not os.path.exists(self._shinobi_directory):
            git.Repo.clone_from(self.docker_shinobi_git_repo_url, self._shinobi_directory,
                                branch=self.docker_shinobi_git_repo_branch)


def start_shinobi(*args, **kwargs) -> ShinobiController:
    """
    Wrapper for starting Shinobi easily in a context, e.g.
    ```
    with start_shinobi() as shinobi:
        print(shinobi)
    ```
    :param args: positional arguments to pass to `ShinobiController.__init__`
    :param kwargs: key word arguments to pass to `ShinobiController.__init__`
    :return: created Shinboi controller
    """
    return ShinobiController(*args, **kwargs)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['shinobi_client', 'shinobi_client.tests']

package_data = \
{'': ['*']}

install_requires = \
['logzero>=1.5.0,<2.0.0', 'requests>=2.23,<3.0', 'toml>=0.9,<0.10']

extras_require = \
{'cli': ['fire>=0.3.0,<0.4.0'],
 'shinobi-controller': ['gitpython>=3,<4',
                        'docker-compose>=1.25,<2.0',
                        'get-port>=0.0.5,<0.0.6']}

setup_kwargs = {
    'name': 'shinobi-client',
    'version': '0.2.0',
    'description': 'A Python client for controlling Shinobi (an open-source video management solution)',
    'long_description': '[![Build Status](https://travis-ci.com/colin-nolan/python-shinobi-client.svg?branch=master)](https://travis-ci.com/colin-nolan/python-shinobi-client)\n[![Code Coverage](https://codecov.io/gh/colin-nolan/python-shinobi-client/branch/master/graph/badge.svg)](https://codecov.io/gh/colin-nolan/python-shinobi-client)\n\n# Shinobi Python Client\n_A Python client for controlling [Shinobi](https://gitlab.com/Shinobi-Systems/Shinobi) (an open-source video management \nsolution)._\n\n\n## About\nThis package contains an (very incomplete) set of tools for interacting with Shinobi using Python.\n\nThis library tries to use the (rather unique) [documented API](https://shinobi.video/docs/api) but it also uses \nundocumented endpoints (which may not be stable).\n\n\n## Installation\nInstall from [PyPi](https://pypi.org/project/shinobi-client/):\n```\npip install shinobi-client\n```\n\nInstall with ability to start a Shinobi installation:\n```\npip install shinobi-client[shinobi-controller]\n```\n\nInstall with CLI:\n```\npip install shinobi-client[cli]\n```\n\n## Usage\n### Python\n#### User ORM\n```python\nfrom shinobi_client import ShinobiUserOrm\n\nuser_orm = ShinobiUserOrm(host, port, super_user_token)\n\nuser = user_orm.get(email)\n\nusers = user_orm.get_all()\n\nuser = user_orm.create(email, password)\n\nmodified = user_orm.modify(email, password=new_password)\n\ndeleted = user_orm.delete(email)\n```\n\n#### Shinobi Controller\nStarts/Stops a temporary [containerised installation of Shinboi](https://github.com/colin-nolan/docker-shinobi). Written\nfor the purpose of testing but it is also installable as an extra. Requires Docker.\n```python\nfrom shinobi_client import start_shinobi\n\nwith start_shinobi() as shinobi:\n    print(shinobi.url)\n    # Do things with a temporary Shinobi installation\n```\nor\n```python\nfrom shinobi_client import ShinobiController\n\ncontroller = ShinobiController()\nshinobi = controller.start()\nprint(shinobi.url)\n# Do things with a temporary Shinobi installation\ncontroller.stop()\n```\n\n### CLI\nA basic auto-generated CLI is available if the package is installed with the `cli` extra: \n```\nPYTHONPATH=. python shinobi_client/user.py --host=HOST --port=PORT --super_user_token=SUPER_USER_TOKEN get example@\n```\ne.g.\n```\n$ PYTHONPATH=. python shinobi_client/user.py \\\n        --host=\'0.0.0.0\' --port=50694 --super_user_token=\'26dd3352-73c4-4bbd-8b09-17f2aacbd7b9\' \\\n    create \'user@example.com\' \'password123\'\n```\n\n\n## Development\nInstall with dev-dependencies:\n```\npoetry install --no-root --extras "shinobi-controller"\n```\n\nRun tests with:\n```\npython -m unittest discover -v -s shinobi/tests\n```\n\n\n## Legal\n[AGPL v3.0](LICENSE.txt). Copyright 2020 Colin Nolan.\n\nI am not affiliated to the development of Shinobi project in any way. This work is in no way related to the company that\nI work for.\n',
    'author': 'Colin Nolan',
    'author_email': 'cn580@alumni.york.ac.uk',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/colin-nolan/python-shinobi-client',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
